/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/mode.c by iffe version 2024-03-20 : : */
#ifndef _def_mode_ast
#define _def_mode_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define S_ITYPE(m)	((m)&S_IFMT)

#define S_IPERM		(S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO)

#define _S_IDPERM	1
#define _S_IDTYPE	1

#define BUFFERSIZE	65536

#endif
