/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/limits.c by iffe version 2024-03-20 : : */
#ifndef _def_limits_ast
#define _def_limits_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */


#ifndef ARG_MAX
#define ARG_MAX 524288
#endif
#ifndef CHARCLASS_NAME_MAX
#define CHARCLASS_NAME_MAX 14
#endif
#ifndef CHAR_BIT
#define CHAR_BIT 8
#endif
#ifndef CHAR_MAX
#define CHAR_MAX 127
#endif
#ifndef CHAR_MIN
#define CHAR_MIN -128
#endif
#ifndef CHILD_MAX
#define CHILD_MAX 256
#endif
#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC 100
#endif
#ifndef FCHR_MAX
#ifndef SSIZE_MAX
#define SSIZE_MAX 9223372036854775807
#endif
#define FCHR_MAX SSIZE_MAX
#endif
#ifndef INT_MAX
#define INT_MAX 2147483647
#endif
#ifndef INT_MIN
#define INT_MIN -2147483648
#endif
#ifndef LLONG_MAX
#define LLONG_MAX 9223372036854775807
#endif
#ifndef LLONG_MIN
#define LLONG_MIN -9223372036854775808
#endif
#ifndef LONG_BIT
#define LONG_BIT 64
#endif
#ifndef LONG_MAX
#define LONG_MAX 9223372036854775807
#endif
#ifndef LONG_MIN
#define LONG_MIN -9223372036854775808
#endif
#ifndef MB_LEN_MAX
#define MB_LEN_MAX 4
#endif
#ifndef NL_ARGMAX
#define NL_ARGMAX 9
#endif
#ifndef NL_LANGMAX
#define NL_LANGMAX 14
#endif
#ifndef NL_MSGMAX
#define NL_MSGMAX 32767
#endif
#ifndef NL_NMAX
#define NL_NMAX 1
#endif
#ifndef NL_SETMAX
#define NL_SETMAX 255
#endif
#ifndef NL_TEXTMAX
#ifndef LINE_MAX
#define LINE_MAX 2048
#endif
#define NL_TEXTMAX LINE_MAX
#endif
#ifndef NZERO
#define NZERO 20
#endif
#ifndef OPEN_MAX
#define OPEN_MAX 512
#endif
#ifndef OPEN_MAX_CEIL
#ifndef OPEN_MAX
#define OPEN_MAX 64
#endif
#define OPEN_MAX_CEIL OPEN_MAX
#endif
#ifndef PASS_MAX
#define PASS_MAX 8
#endif
#ifndef PATH_MAX
#define PATH_MAX 1024
#endif
#ifndef PID_MAX
#define PID_MAX 99999
#endif
#ifndef PTRDIFF_MAX
#define PTRDIFF_MAX 9223372036854775807
#endif
#ifndef PTRDIFF_MIN
#define PTRDIFF_MIN -9223372036854775808
#endif
#ifndef SCHAR_MAX
#define SCHAR_MAX 127
#endif
#ifndef SCHAR_MIN
#define SCHAR_MIN -128
#endif
#ifndef SHRT_MAX
#define SHRT_MAX 32767
#endif
#ifndef SHRT_MIN
#define SHRT_MIN -32768
#endif
#ifndef SIG_ATOMIC_MAX
#define SIG_ATOMIC_MAX 2147483647
#endif
#ifndef SIG_ATOMIC_MIN
#define SIG_ATOMIC_MIN -2147483648
#endif
#ifndef SIZE_MAX
#define SIZE_MAX 18446744073709551615
#endif
#ifndef SSIZE_MAX
#define SSIZE_MAX 9223372036854775807
#endif
#ifndef STD_BLK
#define STD_BLK 1024
#endif
#ifndef SYSPID_MAX
#define SYSPID_MAX 2
#endif
#ifndef TMP_MAX
#define TMP_MAX 2147483647
#endif
#ifndef UCHAR_MAX
#define UCHAR_MAX 255
#endif
#ifndef UID_MAX
#define UID_MAX 4294967295
#endif
#ifndef UINT_MAX
#define UINT_MAX 4294967295
#endif
#ifndef ULLONG_MAX
#define ULLONG_MAX 18446744073709551615
#endif
#ifndef ULONG_MAX
#define ULONG_MAX 18446744073709551615
#endif
#ifndef USHRT_MAX
#define USHRT_MAX 65535
#endif
#ifndef WCHAR_MAX
#define WCHAR_MAX 2147483647
#endif
#ifndef WCHAR_MIN
#define WCHAR_MIN -2147483648
#endif
#ifndef WINT_MAX
#define WINT_MAX 2147483647
#endif
#ifndef WINT_MIN
#define WINT_MIN -2147483648
#endif
#ifndef WORD_BIT
#define WORD_BIT 32
#endif

#endif
