/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/tty by iffe version 2024-03-20 : : */
#ifndef _def_tty_ast
#define _def_tty_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _hdr_termios	1	/* #include <termios.h> ok */
#define _lib_tcgetattr	1	/* tcgetattr() in default lib(s) */
#define _hdr_unistd	1	/* #include <unistd.h> ok */
#define _lib_tcgetpgrp	1	/* tcgetpgrp() in default lib(s) */
#define _mac__POSIX_VDISABLE	1	/* _POSIX_VDISABLE is a macro */

#if _mac__POSIX_VDISABLE
#   undef _POSIX_VDISABLE
#endif
#include <termios.h>
#include <ast_lib.h>

#undef tcgetattr
#undef tcsetattr
#undef tcgetpgrp
#undef tcsetpgrp
#undef cfgetospeed
#ifndef TCSANOW
#   define TCSANOW	TCSETS
#   define TCSADRAIN	TCSETSW
#   define TCSAFLUSH	TCSETSF
#endif /* TCSANOW */
/* The following corrects bugs in some implementations */
#if defined(TCSADFLUSH) && !defined(TCSAFLUSH)
#   define TCSAFLUSH	TCSADFLUSH
#endif /* TCSADFLUSH */
#undef TIOCGETC

#if _sys_ioctl
#   include <sys/ioctl.h>
#endif

/* set ECHOCTL if driver can echo control characters as ^c */
#ifdef LCTLECH
#   ifndef ECHOCTL
#	define ECHOCTL	LCTLECH
#   endif /* !ECHOCTL */
#endif /* LCTLECH */
#ifdef LNEW_CTLECH
#   ifndef ECHOCTL
#	define ECHOCTL  LNEW_CTLECH
#   endif /* !ECHOCTL */
#endif /* LNEW_CTLECH */
#ifdef LNEW_PENDIN
#   ifndef PENDIN
#	define PENDIN LNEW_PENDIN
#  endif /* !PENDIN */
#endif /* LNEW_PENDIN */


#endif
