/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/ccode by iffe version 2024-03-20 : : */
#ifndef _def_ccode_ast
#define _def_ccode_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */

#define CC_ASCII	1		/* ISO-8859-1			*/
#define CC_EBCDIC_E	2		/* X/Open dd(1) EBCDIC		*/
#define CC_EBCDIC_I	3		/* X/Open dd(1) IBM		*/
#define CC_EBCDIC_O	4		/* IBM-1047 MVS OpenEdition	*/
#define CC_EBCDIC_S	5		/* Siemens POSIX-bc		*/
#define CC_EBCDIC_H	6		/* IBM-37 AS/400		*/
#define CC_EBCDIC_M	7		/* IBM MVS COBOL		*/
#define CC_EBCDIC_U	8		/* Micro Focus COBOL		*/

#define CC_MAPS		8		/* number of code maps		*/

#define CC_EBCDIC	CC_EBCDIC_E
#define CC_EBCDIC1	CC_EBCDIC_E
#define CC_EBCDIC2	CC_EBCDIC_I
#define CC_EBCDIC3	CC_EBCDIC_O

#define CC_NATIVE	CC_ASCII	/* native character code	*/
#define CC_ALIEN	CC_EBCDIC	/* alien character code		*/

#define CC_bel		0007		/* bel character		*/
#define CC_esc		0033		/* esc character		*/
#define CC_sub		0032		/* sub character		*/
#define CC_vt		0013		/* vt character			*/
#endif
