/*
* -lcmd function list -- define your own CMDLIST()
*/

CMDLIST(basename)
CMDLIST(cat)
CMDLIST(chgrp)
CMDLIST(chmod)
CMDLIST(chown)
CMDLIST(cksum)
CMDLIST(cmp)
CMDLIST(comm)
CMDLIST(cp)
CMDLIST(cut)
CMDLIST(date)
CMDLIST(dirname)
CMDLIST(expr)
CMDLIST(fds)
CMDLIST(fmt)
CMDLIST(fold)
CMDLIST(getconf)
CMDLIST(head)
CMDLIST(id)
CMDLIST(join)
CMDLIST(ln)
CMDLIST(logname)
CMDLIST(md5sum)
CMDLIST(mkdir)
CMDLIST(mkfifo)
CMDLIST(mktemp)
CMDLIST(mv)
CMDLIST(paste)
CMDLIST(pathchk)
CMDLIST(pids)
CMDLIST(rev)
CMDLIST(rm)
CMDLIST(rmdir)
CMDLIST(stty)
CMDLIST(sum)
CMDLIST(sync)
CMDLIST(tail)
CMDLIST(tee)
CMDLIST(tty)
CMDLIST(uname)
CMDLIST(uniq)
CMDLIST(wc)
