/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/param.sh by iffe version 2024-03-20 : : */
#ifndef _def_param_ast
#define _def_param_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#include <sys/param.h>
#ifndef S_IFDIR
#include <sys/stat.h>
#endif
#endif
