/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/api by iffe version 2024-03-20 : : */
#ifndef _AST_API_H
#define _AST_API_H	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _AST_VERSION AST_VERSION /* pre-20100601 compatibility */

#define AST_VERSION	20240303
#define ASTAPI(rel)	( _BLD_ast || !_API_ast || _API_ast >= rel )

#if !defined(_API_ast) && defined(_API_DEFAULT)
#define _API_ast	_API_DEFAULT
#endif

#if ASTAPI(20120411)
#undef	cmdopen
#define cmdopen	cmdopen_20120411
#elif _API_ast >= 20110505
#undef	cmdopen
#define cmdopen	cmdopen_20110505
#endif

#if ASTAPI(20100601)
#undef	pathaccess
#define pathaccess	pathaccess_20100601
#endif

#if ASTAPI(20100601)
#undef	pathcanon
#define pathcanon	pathcanon_20100601
#endif

#if ASTAPI(20100601)
#undef	pathcat
#define pathcat	pathcat_20100601
#endif

#if ASTAPI(20100601)
#undef	pathkey
#define pathkey	pathkey_20100601
#endif

#if ASTAPI(20100601)
#undef	pathpath
#define pathpath	pathpath_20100601
#endif

#if ASTAPI(20100601)
#undef	pathrepl
#define pathrepl	pathrepl_20100601
#endif

#if ASTAPI(20120528)
#undef	regexec
#define regexec	regexec_20120528
#endif

#if ASTAPI(20120528)
#undef	regnexec
#define regnexec	regnexec_20120528
#endif

#if ASTAPI(20120528)
#undef	regrexec
#define regrexec	regrexec_20120528
#endif

#if ASTAPI(20120528)
#undef	regsubexec
#define regsubexec	regsubexec_20120528
#endif

#if ASTAPI(20000308)
#undef	sfkeyprintf
#define sfkeyprintf	sfkeyprintf_20000308
#endif

#define _API_ast_MAP	"cmdopen_20120411 cmdopen_20110505 pathaccess_20100601 pathcanon_20100601 pathcat_20100601 pathkey_20100601 pathpath_20100601 pathrepl_20100601 regexec_20120528 regnexec_20120528 regrexec_20120528 regsubexec_20120528 sfkeyprintf_20000308"

#endif
