/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/sig.sh by iffe version 2024-03-20 : : */
#ifndef _def_sig_ast
#define _def_sig_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define sig_info	_sig_info_

#define kill	______kill
#define killpg	______killpg
#include <signal.h>
#undef	kill
#undef	killpg
#ifndef sigmask
#define sigmask(s)	(1<<((s)-1))
#endif
typedef void (*Sig_handler_t)(int);


#define Handler_t		Sig_handler_t

#define SIG_REG_PENDING		(-1)
#define SIG_REG_POP		0
#define SIG_REG_EXEC		00001
#define SIG_REG_PROC		00002
#define SIG_REG_TERM		00004
#define SIG_REG_ALL		00777
#define SIG_REG_SET		01000

typedef struct
{
	char**		name;
	char**		text;
	int		sigmax;
} Sig_info_t;

extern int		kill(pid_t, int);
extern int		killpg(pid_t, int);

extern Sig_info_t	sig_info;

#if _lib_sigflag && _npt_sigflag
extern int		sigflag(int, int, int);
#endif

#if !_lib_sigflag
extern int		sigflag(int, int, int);
#endif
extern int		sigcritical(int);
extern int		sigunblock(int);
extern Sig_handler_t	_ast_signal(int, Sig_handler_t);
#endif
