/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/standards by iffe version 2024-03-20 : : */
#ifndef _def_standards_ast
#define _def_standards_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
/* BSD (Free, Net, Open, et al) */
	/* No standards or features macro here. On BSD, everything is enabled by default */

#endif
